################################################################################
# File:             createTransformedCurves.R
# Created by:       Pierre Lafaye De Micheaux
# First released:   06.11.2018
# Last revised:     06.11.2018
# 
# R-script for the transformation of all curves to a single coordinate system.
# 
# For a detailed description see:
#   Lafaye De Micheaux, P., Mozharovskyi, P. and Vimond, M. (2018). Depth for 
#     curve data and applications.
################################################################################


apply.best.transf <- function(givencurve, best.par) {
 phi <- best.par[1]
 theta <- best.par[2]
 psi <- best.par[3]
 b <- sin(-phi / 2) * cos(-theta / 2) * cos(-psi / 2) - cos(-phi / 2) * sin(-theta / 2) * sin(-psi / 2)
 c <- cos(-phi / 2) * sin(-theta / 2) * cos(-psi / 2) + sin(-phi / 2) * cos(-theta / 2) * sin(-psi / 2)
 d <- cos(-phi / 2) * cos(-theta / 2) * sin(-psi / 2) - sin(-phi / 2) * sin(-theta / 2) * cos(-psi / 2)
 bcd <- c(b, c, d)
 xyz <- best.par[4:6]

    RphiX <- matrix(c( 1,        0,        0, 
                   0,        cos(phi), -sin(phi),
                   0,        sin(phi),  cos(phi)),
                nrow = 3, byrow = TRUE)
  RthetaY <- matrix(c( cos(theta),  0,        sin(theta),
                   0,        1,        0,
                  -sin(theta),  0,        cos(theta)),
                nrow = 3, byrow = TRUE)
  RpsiZ <- matrix(c( cos(psi), -sin(psi),  0,
                   sin(psi),  cos(psi),  0,
                   0,        0,        1),
                nrow = 3, byrow = TRUE)
  Ra <- RphiX %*% RthetaY %*% RpsiZ

 # There is some problem with the sign of the quaternions, so I do not use them anymore ...

 centre <- colMeans(givencurve$coords)
 centre.mat <- matrix(rep(centre, nrow(givencurve$coords)), 
                       nrow = nrow(givencurve$coords), byrow = TRUE)
 qoffsetxyz <- matrix(rep(xyz, nrow(givencurve$coords)), 
                       nrow = nrow(givencurve$coords), byrow = TRUE)
 givencurve$coords <- (givencurve$coords  - centre.mat) %*% t(Ra)  + centre.mat + qoffsetxyz

#  givencurve$coords <- (givencurve$coords  - centre.mat) %*% Q2R(bcd, 1)  + centre.mat + qoffsetxyz

# givencurve$coords <- (givencurve$coords  - centre.mat + (centre.mat + qoffsetxyz) %*% t(Q2R(bcd, 1))) %*% Q2R(bcd, 1)  
 
 return(givencurve)
}

X <- read.csv("oats_sydney_w2_info-clean.csv")
codes <- X$OATSID

require("AnalyzeFMRI")
source("../DTI.R")

for (code in codes) {
  filename.given <- paste(code, "_1_dwi.voxelcoordsL.txt", sep = "")
  # Read fibers file filename.given
  fibers.Left <- f.read.fibers(fibers.file = paste("../Data/", filename.given, sep = ""))
  fibers.Left.coord <- fibers.Left$fibers.coordinates
  fibers.Left.nb <- fibers.Left$nbfibers
  fibers.Left.min.nbpoints <- min(fibers.Left$vect.of.nbpts.per.fiber)
  # Number of curves taken from each side of the brain.
  # It is 1,000 for this specific example
  n <- fibers.Left.nb
  # Create curve objects for the fibers of the left side of the brain
  best.par <- scan(paste("../Step4/transform.par.", code, ".to.deepest.of.all.txt", sep = ""), n = 6)
  leftFibers <- list("")
  for (j in 1:n) {
    f1 <- fibers.Left$indices.of.fibers[j, 1]
    f2 <- fibers.Left$indices.of.fibers[j, 2]
    x <- fibers.Left.coord[f1:f2, 1]
    y <- fibers.Left.coord[f1:f2, 2]
    z <- fibers.Left.coord[f1:f2, 3]
    leftFibers[[j]] <- list(voxels = 0, coords = cbind(x, y, z))
    leftFibers[[j]] <- apply.best.transf(leftFibers[[j]], best.par)
  }

  eval(parse(text = paste("leftFibers", code, " <- leftFibers", sep = "")))
  filename <- paste("TransformedCurves/transformed-curve-", code, ".RData", sep = "")
  eval(parse(text = paste("save(leftFibers", code, ", file = filename)", sep = "")))

}


